%% Initialisation et paramètres du modèle géostatistique
% Ce bloc de code initialise les paramètres nécessaires à la simulation
% géostatistique et définit les dimensions du champ de simulation.

% --- Taille du champ et définition de la grille de simulation ---
nx = 100;   % Nombre de cellules en direction x
ny = 100;   % Nombre de cellules en direction y

% Génération de la grille 2D
x0 = grille2(1, nx, 1, 1, ny, 1);  % Appel de la fonction pour générer la grille

% --- Paramètres de simulation ---
nbsim = 1;     % Nombre de réalisations (simulations)
seed = 89456;  % Graine pour la génération aléatoire (pour assurer la reproductibilité)
c = 0.25;      % Sill du variogramme (valeur à laquelle le variogramme atteint son maximum)

% --- Paramètres du modèle géostatistique ---
model = [4, 25, 25, 0];  % Modèle géostatistique isotrope sphérique avec une portée (range) de 25 mètres

FlowParameters.nx    = nx;
FlowParameters.ny    = ny;
FlowParameters.BC    = [0 ; 1];  % Conditions aux limites
FlowParameters.Kmean = 100;      % Conductivité moyenne

% Le modèle représente un variogramme sphérique isotrope de portée 25 mètres.

%% Calcul du champ de conductivité hydraulique de référence et des charges hydrauliques
Reference(model, c, seed, nx, ny);  % Appel de la fonction pour calculer les champs de référence

%% Données observées
% Ce bloc définit les localisations des piézomètres et extrait les données observées

% --- Localisation des piézomètres ---
x0Piezo(:,1) = nx/2 + [8 ; -16 ; -8  ;  16 ; 4 ; -4 ;...
                        0  ;   0 ;  0  ;  0 ; 0 ; 0  ;...
                        24 ;  24 ; -24 ; -24  ;...
                        0  ;   0 ;  24 ; -24];  % Coordonnées en x

x0Piezo(:,2) = ny/2 + [0 ;   0 ;   0 ;  0;  0 ; 0  ;...
                        8 ; -16 ;  -8 ;  16; 4 ; -4 ;...
                        24 ; -24 ; -24 ;  24 ;...
                        24 ; -24 ;   0 ;   0];  % Coordonnées en y

% --- Index des piézomètres ---
LocData = nx * (x0Piezo(:,2) - 1) + x0Piezo(:,1);

% --- Chargement des données de référence ---
load('Reference_Data.mat', 'Zref', 'Rref');

% --- Extraction des données observées ---
D = Rref(LocData);  % Extraction des valeurs observées aux piézomètres

%% Champs de conductivité hydraulique initiale
% 100 réalisations indépendantes.

seed = 89456;  % Graine pour la reproductibilité

%%%%%% Paramètres à modifier pour répondre aux questions %%%%%%%%%%%%%
model = [4, 25, 25, 0]; c= 0.25;
nbsim = 100;  % Nombre de réalisations

% --- Génération des réalisations ---
E = fftma(model, c, seed, nbsim, nx, 1, ny, 1);  % Génération de 100 réalisations du champ

% Charge hydraulique initiale du champ E final
yinit = zeros(20, nbsim);  % Stockage des charges assimilées
for i = 1:nbsim
    FlowParameters.Z = E(:,i);  % Mise à jour des paramètres du modèle
    Rsim = FlowSimulation(FlowParameters);  % Simulation de l'écoulement
    yinit(:,i) = Rsim(LocData);  % Stockage des charges simulées
end

% Initialisation de la variable y pour stocker les résultats
y = zeros(20, nbsim);  % Stockage des charges assimilées

for j = 1 : 20
    j
    % Calcul des charges pour chaque simulation
    for i = 1:nbsim
        FlowParameters.Z = E(:,i);  % Mise à jour des paramètres du modèle
        Rsim = FlowSimulation(FlowParameters);  % Simulation de l'écoulement
        y(:,i) = Rsim(LocData);  % Stockage des charges simulées
    end
    
    % Matrice d'anomalie
    A = E - mean(E, 2);  % Anomalie des paramètres simulés
    Y = y - mean(y, 2);  % Anomalie des observations

    % Calcul de la matrice de gain de Kalman
    R = diag(ones(20,1)) * 0.05;  % Matrice des erreurs de mesure
    K = A * Y' / (Y * Y' + R);  % Gain de Kalman

    % Mise à jour de la matrice E
    E = E + K * (D - y);  % Mise à jour des réalisations
end

% Charge hydraulique finale du champ E final
for i = 1:nbsim
    FlowParameters.Z = E(:,i);  % Mise à jour des paramètres du modèle
    Rsim = FlowSimulation(FlowParameters);  % Simulation de l'écoulement
    y(:,i) = Rsim(LocData);  % Stockage des charges simulées
end

%% Visualisation sous forme de boxplot pour 20 piézomètres
figure(153);
hold on;

% Initialisation de la matrice pour les boxplots
boxData = NaN(20, 100);  % 20 piézomètres, 100 pour les simulations uniquement

% Remplir les données pour chaque piézomètre (uniquement les simulations)
for pi = 1:20
    % Remplir les données du boxplot pour ce piézomètre (simulations)
    boxData(pi, :) = yinit(pi,:);  % Simulations (les 100 réalisations)
end

% Création du boxplot pour chaque piézomètre (20 boxplots en tout)
boxplot(boxData', 'Labels', num2str((1:20)'),'Whisker', 1.5);

% Affichage des valeurs observées et de la moyenne assimilée sous forme de points
for pi = 1:20
    % Valeur observée (en rouge)
    observedData = D(pi);
    plot(pi, observedData, 'ko', 'MarkerFaceColor', 'r', 'MarkerSize', 6);  % Point rouge
    
    % Moyenne assimilée (en noir)
    assimilatedMean = mean(y(pi,:));
    plot(pi, assimilatedMean, 'ko', 'MarkerFaceColor', 'k', 'MarkerSize', 6);  % Point noir
end

% Titres et légendes
ylabel('Charge hydraulique (m)');  % Légende de l'axe des y
title('Boxplots : Charges observées, assimilées et simulées pour 20 piézomètres');  % Titre du graphique
grid on;

% Ajuster l'affichage pour plus de lisibilité
set(gca, 'XTickLabel', 1:20); % Afficher l'indice des piézomètres
xtickangle(90);  % Rotation des labels des piézomètres pour éviter le chevauchement

% Légende des éléments de boxplot
legend({'Observé', 'Moyenne initiale', 'Simulations'}, 'Location', 'Best');

figure(152);
hold on;

% Initialisation de la matrice pour les boxplots
boxData = NaN(20, 100);  % 20 piézomètres, 100 pour les simulations uniquement

% Remplir les données pour chaque piézomètre (uniquement les simulations)
for pi = 1:20
    % Remplir les données du boxplot pour ce piézomètre (simulations)
    boxData(pi, :) = y(pi,:);  % Simulations (les 100 réalisations)
end

% Création du boxplot pour chaque piézomètre (20 boxplots en tout)
boxplot(boxData', 'Labels', num2str((1:20)'),'Whisker', 1.5);

% Affichage des valeurs observées et de la moyenne assimilée sous forme de points
for pi = 1:20
    % Valeur observée (en rouge)
    observedData = D(pi);
    plot(pi, observedData, 'ko', 'MarkerFaceColor', 'r', 'MarkerSize', 6);  % Point rouge
    
    % Moyenne assimilée (en noir)
    assimilatedMean = mean(y(pi,:));
    plot(pi, assimilatedMean, 'ko', 'MarkerFaceColor', 'k', 'MarkerSize', 6);  % Point noir
end

% Titres et légendes
ylabel('Charge hydraulique (m)');  % Légende de l'axe des y
title('Boxplots : Charges observées, assimilées et simulées pour 20 piézomètres');  % Titre du graphique
grid on;

% Ajuster l'affichage pour plus de lisibilité
set(gca, 'XTickLabel', 1:20); % Afficher l'indice des piézomètres
xtickangle(90);  % Rotation des labels des piézomètres pour éviter le chevauchement

% Légende des éléments de boxplot
legend({'Observé', 'Moyenne assimilée', 'Simulations'}, 'Location', 'Best');
